import json
from typing import IO, Any, Dict, List, Optional, Tuple

from django.http import HttpResponse
from django.test import Client, TestCase


class BaseGraphQLQueryTest(TestCase):
    @staticmethod
    def get_client():
        return Client()

    @classmethod
    def get_operations(
        cls,
        query: str,
        variables: Optional[Dict] = None,
        operation_name: Optional[str] = None,
        input_data: Optional[Dict] = None,
    ) -> Dict:
        if variables is None:
            variables = {}

        if input_data:
            variables["input"] = input_data

        operations = {
            "query": query,
            "variables": variables,
        }
        if operation_name:
            operations["operationName"] = operation_name

        return operations

    @classmethod
    def get_file_map_data(cls, files: List[IO]) -> Tuple[Dict, Dict]:
        file_map = {}
        map_data = {}
        for count, file in enumerate(files):
            map_data[f"{count}"] = ["variables.file"]
            file_map[count] = file

        return map_data, file_map

    @classmethod
    def query(
        cls,
        query: str,
        client: Optional[Client] = None,
        operation_name: Optional[str] = None,
        input_data: Optional[Dict] = None,
        variables: Optional[Dict] = None,
        headers: Optional[Dict] = None,
        files: Optional[List[IO]] = None,
        url: str = "/graphql/",
    ) -> HttpResponse:
        if client is None:
            client = cls.get_client()

        if headers is None:
            headers = {}

        if files is None:
            files = []

        arguments: Dict[str, Any] = {}
        operations_data = cls.get_operations(query, variables, operation_name, input_data)
        if files:
            operations_data["variables"]["file"] = None
            map_data, file_map = cls.get_file_map_data(files)

            data: Any = {"operations": json.dumps(operations_data), "map": json.dumps(map_data), **file_map}
            arguments["format"] = "multipart"
        else:
            data = json.dumps(operations_data)
            arguments["format"] = "json"
            arguments["content_type"] = "application/json"

        rsp = client.post(url, data, **headers, **arguments)
        return rsp



import uuid

from ariadne import ScalarType


uuid_scalar = ScalarType("UUID")


@uuid_scalar.value_parser
def parse_uuid_value(value):
    return uuid.UUID(value)


@uuid_scalar.serializer
def serialize_uuid(value):
    return str(value)



import json
from typing import Any

from ariadne import ScalarType


json_scalar = ScalarType("JSON")


@json_scalar.serializer
def serialize_json(value: Any) -> str:
    return json.dumps(value)


@json_scalar.value_parser
def parse_json_value(value: str) -> Any:
    return json.loads(value)



from django.core.exceptions import PermissionDenied


def permission_required(required_permission):
    def wrapped_decorator(func):
        def wrapped(cls, info, *args, **kwargs):
            perms = (required_permission,) if isinstance(required_permission, str) else required_permission
            user = info.context["request"].user
            if not user.has_perms(perms):
                raise PermissionDenied()
            return func(cls, info, *args, **kwargs)

        return wrapped

    return wrapped_decorator



import json
from typing import Any, Callable, Optional, Union

from django.conf import settings
from django.http import HttpRequest
from django.shortcuts import render
from django.views.generic import View
from django.views.generic.base import ContextMixin, TemplateResponseMixin

from ariadne.constants import DATA_TYPE_JSON, DATA_TYPE_MULTIPART
from ariadne.exceptions import HttpBadRequestError
from ariadne.file_uploads import combine_multipart_data
from ariadne.format_error import format_error
from ariadne.types import ContextValue, ErrorFormatter, ExtensionList, RootValue, ValidationRules

from graphql import GraphQLSchema
from graphql.execution import MiddlewareManager


Extensions = Union[Callable[[Any, Optional[ContextValue]], ExtensionList], ExtensionList]

# https://github.com/graphql/graphql-playground#properties
# For complete fields list see PlaygroundWrapperProps interface
DEFAULT_PLAYGROUND_OPTIONS = {
    # Playground settings
    "settings": {
        "request.credentials": "same-origin",
    },
    # Request HTTP headers added by default
    "headers": {},
}


class BaseGraphQLView(TemplateResponseMixin, ContextMixin, View):
    http_method_names = ["get", "post", "options"]
    template_name = "ariadne_django/graphql_playground.html"
    playground_options: Optional[dict] = None
    introspection: bool = True
    schema: Optional[GraphQLSchema] = None
    context_value: Optional[ContextValue] = None
    root_value: Optional[RootValue] = None
    logger = None
    validation_rules: Optional[ValidationRules] = None
    error_formatter: Optional[ErrorFormatter] = None
    extensions: Optional[Extensions] = None
    middleware: Optional[MiddlewareManager] = None

    def _get(self, request: HttpRequest, *args, **kwargs):  # pylint: disable=unused-argument
        options = DEFAULT_PLAYGROUND_OPTIONS.copy()
        if self.playground_options:
            options.update(self.playground_options)

        return render(
            request,
            self.get_template_names(),
            {"playground_options": json.dumps(options)},
        )

    def extract_data_from_request(self, request: HttpRequest):
        content_type = request.content_type or ""
        content_type = content_type.split(";")[0]

        if content_type == DATA_TYPE_JSON:
            return self.extract_data_from_json_request(request)
        if content_type == DATA_TYPE_MULTIPART:
            return self.extract_data_from_multipart_request(request)

        raise HttpBadRequestError("Posted content must be of type {} or {}".format(DATA_TYPE_JSON, DATA_TYPE_MULTIPART))

    def extract_data_from_json_request(self, request: HttpRequest):
        try:
            return json.loads(request.body)
        except (TypeError, ValueError) as ex:
            raise HttpBadRequestError("Request body is not a valid JSON") from ex

    def extract_data_from_multipart_request(self, request: HttpRequest):
        try:
            operations = json.loads(request.POST.get("operations", "{}"))
        except (TypeError, ValueError) as ex:
            raise HttpBadRequestError("Request 'operations' multipart field is not a valid JSON") from ex
        try:
            files_map = json.loads(request.POST.get("map", "{}"))
        except (TypeError, ValueError) as ex:
            raise HttpBadRequestError("Request 'map' multipart field is not a valid JSON") from ex

        return combine_multipart_data(operations, files_map, request.FILES)

    def get_kwargs_graphql(self, request: HttpRequest) -> dict:
        context_value = self.get_context_for_request(request)
        extensions = self.get_extensions_for_request(request, context_value)

        return {
            "context_value": context_value,
            "root_value": self.root_value,
            "validation_rules": self.validation_rules,
            "debug": settings.DEBUG,
            "introspection": self.introspection,
            "logger": self.logger,
            "error_formatter": self.error_formatter or format_error,
            "extensions": extensions,
            "middleware": self.middleware,
        }

    def get_context_for_request(self, request: HttpRequest) -> Optional[ContextValue]:
        if callable(self.context_value):
            return self.context_value(request)  # pylint: disable=not-callable
        return self.context_value or {"request": request}

    def get_extensions_for_request(self, request: HttpRequest, context: Optional[ContextValue]) -> ExtensionList:
        if callable(self.extensions):
            return self.extensions(request, context)  # pylint: disable=not-callable
        return self.extensions



# flake8: noqa: E501
from .decimal import decimal_scalar, parse_decimal_value, serialize_decimal
from .isodate import date_scalar, parse_date_value, serialize_date
from .isodatetime import datetime_scalar, parse_datetime_value, serialize_datetime
from .isotime import time_scalar, parse_time_value, serialize_time
from .json import json_scalar, parse_json_value, serialize_json
from .uuid import parse_uuid_value, serialize_uuid, uuid_scalar



from django.core.exceptions import PermissionDenied


def login_required():
    def wrapped_decorator(func):
        def wrapped(cls, info, *args, **kwargs):
            user = info.context["request"].user
            if not user.is_authenticated:
                raise PermissionDenied()
            return func(cls, info, *args, **kwargs)

        return wrapped

    return wrapped_decorator



from datetime import datetime
from typing import Any

from django.forms.utils import from_current_timezone
from django.utils import formats
from django.utils.translation import gettext_lazy as _

from ariadne import ScalarType

from ariadne_django.scalars.utils.parsers import parse_value


datetime_input_formats = formats.get_format_lazy("DATETIME_INPUT_FORMATS")
datetime_scalar = ScalarType("DateTime")


@datetime_scalar.serializer
def serialize_datetime(value: datetime) -> str:
    return value.isoformat()


@datetime_scalar.value_parser
def parse_datetime_value(value: Any) -> datetime:
    parsed_value = parse_value(value, datetime_input_formats)
    if not parsed_value:
        raise ValueError(_("Enter a valid date/time."))
    return from_current_timezone(parsed_value)



import asyncio
from typing import cast

from django.http import HttpRequest, HttpResponseBadRequest, JsonResponse
from django.utils.decorators import classonlymethod, method_decorator
from django.views.decorators.csrf import csrf_exempt

from ariadne.exceptions import HttpBadRequestError
from ariadne.graphql import graphql

from graphql import GraphQLSchema

from .base import BaseGraphQLView


@method_decorator(csrf_exempt, name="dispatch")
class GraphQLAsyncView(BaseGraphQLView):
    @classonlymethod
    def as_view(cls, **initkwargs):
        view = super().as_view(**initkwargs)
        view._is_coroutine = asyncio.coroutines._is_coroutine  # pylint: disable=protected-access
        return view

    async def get(self, *args, **kwargs):
        return self._get(*args, **kwargs)

    async def post(self, request: HttpRequest, *args, **kwargs):  # pylint: disable=unused-argument
        try:
            data = self.extract_data_from_request(request)
        except HttpBadRequestError as error:
            return HttpResponseBadRequest(error.message)
        success, result = await graphql(cast(GraphQLSchema, self.schema), data, **self.get_kwargs_graphql(request))
        status_code = 200 if success else 400
        return JsonResponse(result, status=status_code)



from decimal import Decimal, InvalidOperation

from django.utils.formats import number_format, sanitize_separators

from ariadne import ScalarType


decimal_scalar = ScalarType("Decimal")


@decimal_scalar.serializer
def serialize_decimal(value: Decimal) -> str:
    return number_format(value)


@decimal_scalar.value_parser
def parse_decimal_value(value: str) -> Decimal:
    if not isinstance(value, str):
        raise InvalidOperation("Decimal must be a string")
    return Decimal(sanitize_separators(value))



from datetime import timedelta
from typing import Union

from ariadne import ScalarType


timedelta_scalar = ScalarType("Timedelta")


@timedelta_scalar.serializer
def serialize_timedelta(value: timedelta) -> float:
    return value.total_seconds()


@timedelta_scalar.value_parser
def parse_timedelta_value(value: Union[int, float]) -> timedelta:
    return timedelta(seconds=value)



#! /usr/bin/env python
import os

from setuptools import setup


CLASSIFIERS = [
    "Development Status :: 2 - Pre-Alpha",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: BSD License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Topic :: Software Development :: Libraries :: Python Modules",
]

README_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md")
with open(README_PATH, "r", encoding="utf8") as f:
    README = f.read()

setup(
    name="ariadne_django",
    author="Lexria",
    author_email="noreply@lexria.com",
    description="ariadne_django is a python library for integrating django with ariadne.",
    long_description=README,
    long_description_content_type="text/markdown",
    license="BSD",
    version="0.3.0",
    url="https://github.com/reset-button/ariadne_django",
    packages=["ariadne_django"],
    include_package_data=True,
    install_requires=[
        "django>=2.2.0",
        "ariadne>=0.13.0",
    ],
    classifiers=CLASSIFIERS,
    platforms=["any"],
    zip_safe=False,
)



from datetime import datetime
from typing import Any, List, Optional

import dateutil.parser


def parse_value(value: Any, formats: List[str]) -> Optional[datetime]:
    for format_str in formats:
        try:
            return datetime.strptime(value, format_str)
        except (ValueError, TypeError):
            continue

    # fallback to using dateutil parser
    try:
        return dateutil.parser.parse(value)
    except (ValueError, TypeError):
        return None



from datetime import date, datetime
from typing import Any, Union

from django.utils import formats
from django.utils.translation import gettext_lazy as _

from ariadne import ScalarType

from ariadne_django.scalars.utils.parsers import parse_value


date_scalar = ScalarType("Date")
date_input_formats = formats.get_format_lazy("DATE_INPUT_FORMATS")


@date_scalar.serializer
def serialize_date(value: Union[date, datetime]) -> str:
    if isinstance(value, datetime):
        value = value.date()
    return value.isoformat()


@date_scalar.value_parser
def parse_date_value(value: Any) -> date:
    parsed_value = parse_value(value, date_input_formats)
    if not parsed_value:
        raise ValueError(_("Enter a valid date."))
    return parsed_value.date()



from datetime import time
from typing import Any

from django.utils import formats
from django.utils.translation import gettext_lazy as _

from ariadne import ScalarType

from ariadne_django.scalars.utils.parsers import parse_value


time_input_formats = formats.get_format_lazy("TIME_INPUT_FORMATS")
time_scalar = ScalarType("Time")


@time_scalar.serializer
def serialize_time(value: time) -> str:
    return value.isoformat()


@time_scalar.value_parser
def parse_time_value(value: Any) -> time:
    parsed_value = parse_value(value, time_input_formats)
    if not parsed_value:
        raise ValueError(_("Enter a valid time."))
    return parsed_value.time()
